@@ -125,14 +125,12 @@ class GerritChangeSourceBase(base.ChangeSource):
         try:
             event = json.loads(bytes2unicode(line))
         except ValueError:
-            msg = "bad json line: %s"
-            log.msg(msg % line)
+            log.msg("bad json line: {}".format(line))
             return defer.succeed(None)
 
         if not(isinstance(event, dict) and "type" in event):
             if self.debug:
-                msg = "no type in event %s"
-                log.msg(msg % line)
+                log.msg("no type in event {}".format(line))
             return defer.succeed(None)
 
         return self.eventReceived(event)
@@ -140,14 +138,13 @@ class GerritChangeSourceBase(base.ChangeSource):
     def eventReceived(self, event):
         if not (event['type'] in self.handled_events):
             if self.debug:
-                msg = "the event type '%s' is not setup to handle"
-                log.msg(msg % event['type'])
+                log.msg("the event type '{}' is not setup to handle".format(event['type']))
             return defer.succeed(None)
 
         # flatten the event dictionary, for easy access with WithProperties
         def flatten(properties, base, event):
             for k, v in event.items():
-                name = "%s.%s" % (base, k)
+                name = "{}.{}".format(base, k)
                 if name in self.EVENT_PROPERTY_BLACKLIST:
                     continue
                 if isinstance(v, dict):
@@ -158,7 +155,7 @@ class GerritChangeSourceBase(base.ChangeSource):
         properties = {}
         flatten(properties, "event", event)
         properties["event.source"] = self.__class__.__name__
-        func_name = "eventReceived_%s" % event["type"].replace("-", "_")
+        func_name = "eventReceived_{}".format(event["type"].replace("-", "_"))
         func = getattr(self, func_name, None)
         if func is None:
             return self.addChangeFromEvent(properties, event)
@@ -166,7 +163,7 @@ class GerritChangeSourceBase(base.ChangeSource):
         return func(properties, event)
 
     @defer.inlineCallbacks
-    def addChange(self, chdict):
+    def addChange(self, event_type, chdict):
         stampdict = {
             "branch": chdict["branch"],
             "revision": chdict["revision"],
@@ -174,12 +171,13 @@ class GerritChangeSourceBase(base.ChangeSource):
             "patch_comment": chdict["comments"],
             "repository": chdict["repository"],
             "project": chdict["project"],
+            "codebase": '',
         }
 
         stampid, found_existing = yield(
              self.master.db.sourcestamps.findOrCreateId(**stampdict))
 
-        if found_existing:
+        if found_existing and event_type in ("patchset-created", "ref-updated"):
             if self.debug or True:
                 eventstr = "{}/{} -- {}:{}".format(
                     self.gitBaseURL, chdict["project"], chdict["branch"],
@@ -213,20 +211,19 @@ class GerritChangeSourceBase(base.ChangeSource):
         # in the core
         event_change = event["change"]
         if event['type'] in ('patchset-created',):
-            return "%s/%s" % (event_change["branch"],
-                              event_change['number'])
+            return "{}/{}".format(event_change["branch"], event_change['number'])
         return event_change["branch"]
 
     @defer.inlineCallbacks
     def addChangeFromEvent(self, properties, event):
         if "change" not in event:
             if self.debug:
-                log.msg("unsupported event %s" % (event["type"],))
+                log.msg("unsupported event {}".format(event["type"]))
             return defer.returnValue(None)
 
         if "patchSet" not in event:
             if self.debug:
-                log.msg("unsupported event %s" % (event["type"],))
+                log.msg("unsupported event {}".format(event["type"]))
             return defer.returnValue(None)
 
         event = _canonicalize_event(event)
@@ -239,7 +236,7 @@ class GerritChangeSourceBase(base.ChangeSource):
                 patchset=event["patchSet"]["number"]
             )
 
-        yield self.addChange({
+        yield self.addChange(event['type'], {
             'author': _gerrit_user_to_author(event_change["owner"]),
             'project': util.bytes2unicode(event_change["project"]),
             'repository': "{}/{}".format(
@@ -251,6 +248,7 @@ class GerritChangeSourceBase(base.ChangeSource):
             'files': files,
             'category': event["type"],
             'properties': properties})
+        return None
 
     def eventReceived_ref_updated(self, properties, event):
         ref = event["refUpdate"]
@@ -259,11 +257,10 @@ class GerritChangeSourceBase(base.ChangeSource):
         if "submitter" in event:
             author = _gerrit_user_to_author(event["submitter"], author)
 
-        return self.addChange(dict(
+        return self.addChange(event['type'], dict(
             author=author,
             project=ref["project"],
-            repository="%s/%s" % (
-                self.gitBaseURL, ref["project"]),
+            repository="{}/{}".format(self.gitBaseURL, ref["project"]),
             branch=ref["refName"],
             revision=ref["newRev"],
             comments="Gerrit: patchset(s) merged.",
@@ -300,8 +297,7 @@ class GerritChangeSource(GerritChangeSourceBase):
                     identity_file=None,
                     **kwargs):
         if self.name is None:
-            self.name = "GerritChangeSource:%s@%s:%d" % (
-                username, gerritserver, gerritport)
+            self.name = "GerritChangeSource:{}@{}:{}".format(username, gerritserver, gerritport)
         if 'gitBaseURL' not in kwargs:
             kwargs['gitBaseURL'] = "automatic at reconfigure"
         super().checkConfig(**kwargs)
@@ -314,7 +310,7 @@ class GerritChangeSource(GerritChangeSourceBase):
                         name=None,
                         **kwargs):
         if 'gitBaseURL' not in kwargs:
-            kwargs['gitBaseURL'] = "ssh://%s@%s:%s" % (username, gerritserver, gerritport)
+            kwargs['gitBaseURL'] = "ssh://{}@{}:{}".format(username, gerritserver, gerritport)
         self.gerritserver = gerritserver
         self.gerritport = gerritport
         self.username = username
@@ -380,7 +376,8 @@ class GerritChangeSource(GerritChangeSourceBase):
 
         cmd = [
             "ssh",
-            "%s@%s" % (self.username, self.gerritserver),
+            "-o", "BatchMode=yes",
+            "{}@{}".format(self.username, self.gerritserver),
             "-p", str(self.gerritport)
         ]
 
@@ -405,8 +402,8 @@ class GerritChangeSource(GerritChangeSourceBase):
                                        "--files", "--patch-sets")
 
         if self.debug:
-            log.msg("querying gerrit for changed files in change %s/%s: %s" %
-                    (change, patchset, cmd))
+            log.msg("querying gerrit for changed files in change {}/{}: {}".format(change, patchset,
+                                                                                   cmd))
 
         out = yield utils.getProcessOutput(cmd[0], cmd[1:], env=None)
         out = out.splitlines()[0]
@@ -433,9 +430,8 @@ class GerritChangeSource(GerritChangeSourceBase):
         status = ""
         if not self.process:
             status = "[NOT CONNECTED - check log]"
-        msg = ("GerritChangeSource watching the remote "
-               "Gerrit repository %s@%s %s")
-        return msg % (self.username, self.gerritserver, status)
+        return (("GerritChangeSource watching the remote "
+                 "Gerrit repository {}@{} {}").format(self.username, self.gerritserver, status))
 
 
 class GerritEventLogPoller(GerritChangeSourceBase):
@@ -495,7 +491,8 @@ class GerritEventLogPoller(GerritChangeSourceBase):
         if self.debug:
             log.msg("Polling gerrit: {}".format(last_event_formatted).encode("utf-8"))
 
-        res = yield self._http.get("/plugins/events-log/events/", params=dict(t1=last_event_formatted))
+        res = yield self._http.get("/plugins/events-log/events/",
+                                   params=dict(t1=last_event_formatted))
         lines = yield res.content()
         for line in lines.splitlines():
             yield self.lineReceived(line)
@@ -509,7 +506,7 @@ class GerritEventLogPoller(GerritChangeSourceBase):
 
     @defer.inlineCallbacks
     def getFiles(self, change, patchset):
-        res = yield self._http.get("/changes/%s/revisions/%s/files/" % (change, patchset))
+        res = yield self._http.get("/changes/{}/revisions/{}/files/".format(change, patchset))
         res = yield res.content()
 
         res = res.splitlines()[1].decode('utf8')  # the first line of every response is `)]}'`
