@@ -19,7 +19,6 @@ import weakref
 from twisted.application import service
 from twisted.internet import defer
 from twisted.python import log
-from zope.interface import implementer
 
 from buildbot import interfaces
 from buildbot.data import resultspec
@@ -76,7 +75,6 @@ class Builder(util_service.ReconfigurableServiceMixin,
         self.workers = []
 
         self.config = None
-        self.builder_status = None
 
         # Tracks config version for locks
         self.config_version = None
@@ -90,14 +88,7 @@ class Builder(util_service.ReconfigurableServiceMixin,
                 break
         assert found_config, "no config found for builder '{}'".format(self.name)
 
-        # set up a builder status object on the first reconfig
-        if not self.builder_status:
-            self.builder_status = self.master.status.builderAdded(
-                name=builder_config.name,
-                basedir=builder_config.builddir,
-                tags=builder_config.tags,
-                description=builder_config.description)
-
+        old_config = self.config
         self.config = builder_config
         self.config_version = self.master.config_version
 
@@ -106,14 +97,10 @@ class Builder(util_service.ReconfigurableServiceMixin,
         # build.
         builderid = yield self.getBuilderId()
 
-        self.master.data.updates.updateBuilderInfo(builderid,
-                                                   builder_config.description,
-                                                   builder_config.tags)
-
-        self.builder_status.setDescription(builder_config.description)
-        self.builder_status.setTags(builder_config.tags)
-        self.builder_status.setWorkernames(self.config.workernames)
-        self.builder_status.setCacheSize(new_config.caches['Builds'])
+        if self._has_updated_config_info(old_config, builder_config):
+            yield self.master.data.updates.updateBuilderInfo(builderid,
+                                                             builder_config.description,
+                                                             builder_config.tags)
 
         # if we have any workers attached which are no longer configured,
         # drop them.
@@ -121,6 +108,15 @@ class Builder(util_service.ReconfigurableServiceMixin,
         self.workers = [w for w in self.workers
                         if w.worker.workername in new_workernames]
 
+    def _has_updated_config_info(self, old_config, new_config):
+        if old_config is None:
+            return True
+        if old_config.description != new_config.description:
+            return True
+        if old_config.tags != new_config.tags:
+            return True
+        return False
+
     def __repr__(self):
         return "<Builder '%r' at %d>" % (self.name, id(self))
 
@@ -179,10 +175,10 @@ class Builder(util_service.ReconfigurableServiceMixin,
 
     def getBuild(self, number):
         for b in self.building:
-            if b.build_status and b.build_status.number == number:
+            if b.number == number:
                 return b
         for b in self.old_building:
-            if b.build_status and b.build_status.number == number:
+            if b.number == number:
                 return b
         return None
 
@@ -230,7 +226,7 @@ class Builder(util_service.ReconfigurableServiceMixin,
         self.attaching_workers.append(wfb)
 
         try:
-            wfb = yield wfb.attached(worker, commands)
+            yield wfb.attached(worker, commands)
             self.attaching_workers.remove(wfb)
             self.workers.append(wfb)
             return self
@@ -278,7 +274,7 @@ class Builder(util_service.ReconfigurableServiceMixin,
         # don't unnecessarily setup properties for build
         def setupPropsIfNeeded(props):
             if props is not None:
-                return None
+                return props
             props = Properties()
             Build.setupPropertiesKnownBeforeBuildStarts(props, [buildrequest],
                                                         self, workerforbuilder)
@@ -344,22 +340,11 @@ class Builder(util_service.ReconfigurableServiceMixin,
         # another build request.
         workerforbuilder.buildStarted()
 
-        # create the BuildStatus object that goes with the Build
-        bs = self.builder_status.newBuild()
-
-        # let status know
-        self.master.status.build_started(buildrequests[0].id, self.name, bs)
-
-        # start the build. This will first set up the steps, then tell the
-        # BuildStatus that it has started, which will announce it to the world
-        # (through our BuilderStatus object, which is its parent).  Finally it
-        # will start the actual build process.  This is done with a fresh
-        # Deferred since _startBuildFor should not wait until the build is
-        # finished.  This uses `maybeDeferred` to ensure that any exceptions
+        # We put the result of startBuild into a fresh Deferred since _startBuildFor should not
+        # wait until the build is finished.  This uses `maybeDeferred` to ensure that any exceptions
         # raised by startBuild are treated as deferred errbacks (see
         # http://trac.buildbot.net/ticket/2428).
-        d = defer.maybeDeferred(build.startBuild,
-                                bs, workerforbuilder)
+        d = defer.maybeDeferred(build.startBuild, workerforbuilder)
         # this shouldn't happen. if it does, the worker will be wedged
         d.addErrback(log.err, 'from a running build; this is a '
                      'serious error - please file a bug at http://buildbot.net')
@@ -389,7 +374,7 @@ class Builder(util_service.ReconfigurableServiceMixin,
         # which will trigger a check for any now-possible build requests
         # (maybeStartBuilds)
 
-        results = build.build_status.getResults()
+        results = build.results
 
         self.building.remove(build)
         if results == RETRY:
@@ -462,49 +447,3 @@ class Builder(util_service.ReconfigurableServiceMixin,
     @staticmethod
     def _defaultCollapseRequestFn(master, builder, brdict1, brdict2):
         return buildrequest.BuildRequest.canBeCollapsed(master, brdict1, brdict2)
-
-
-@implementer(interfaces.IBuilderControl)
-class BuilderControl:
-
-    def __init__(self, builder, control):
-        self.original = builder
-        self.control = control
-
-    @defer.inlineCallbacks
-    def getPendingBuildRequestControls(self):
-        master = self.original.master
-        # TODO Use DATA API
-        brdicts = yield master.db.buildrequests.getBuildRequests(
-            buildername=self.original.name,
-            claimed=False)
-
-        # convert those into BuildRequest objects
-        buildrequests = []
-        for brdict in brdicts:
-            br = yield buildrequest.BuildRequest.fromBrdict(
-                self.control.master, brdict)
-            buildrequests.append(br)
-
-        # and return the corresponding control objects
-        return [buildrequest.BuildRequestControl(self.original, r)
-            for r in buildrequests]
-
-    def getBuild(self, number):
-        return self.original.getBuild(number)
-
-    def ping(self):
-        if not self.original.workers:
-            return defer.succeed(False)  # interfaces.NoWorkerError
-        dl = []
-        for w in self.original.workers:
-            dl.append(w.ping(self.original.builder_status))
-        d = defer.DeferredList(dl)
-        d.addCallback(self._gatherPingResults)
-        return d
-
-    def _gatherPingResults(self, res):
-        for ignored, success in res:
-            if not success:
-                return False
-        return True
