@@ -14,24 +14,23 @@
 # Copyright Buildbot Team Members
 
 
+from twisted.internet import defer
 from twisted.python import log
 
 from buildbot.process import buildstep
 from buildbot.process import properties
 from buildbot.process import remotecommand
-from buildbot.process.buildstep import LoggingBuildStep
-from buildbot.status.builder import FAILURE
-from buildbot.status.builder import SKIPPED
+from buildbot.process.results import FAILURE
 from buildbot.steps.worker import CompositeStepMixin
 from buildbot.util import bytes2unicode
 
 
-class Source(LoggingBuildStep, CompositeStepMixin):
+class Source(buildstep.BuildStep, CompositeStepMixin):
 
     """This is a base class to generate a source tree in the worker.
     Each version control system has a specialized subclass, and is expected
     to override __init__ and implement computeSourceRevision() and
-    startVC(). The class as a whole builds up the self.args dictionary, then
+    run_vc(). The class as a whole builds up the self.args dictionary, then
     starts a RemoteCommand with those arguments.
     """
 
@@ -45,7 +44,6 @@ class Source(LoggingBuildStep, CompositeStepMixin):
     # if the checkout fails, there's no point in doing anything else
     haltOnFailure = True
     flunkOnFailure = True
-    notReally = False
 
     branch = None  # the default branch, should be set in __init__
 
@@ -181,13 +179,6 @@ class Source(LoggingBuildStep, CompositeStepMixin):
                 "Sourcestep {} does not have a codebase, other sourcesteps do".format(self.name)
             super().setProperty(name, value, source)
 
-    def describe(self, done=False):
-        desc = self.descriptionDone if done else self.description
-        if self.descriptionSuffix:
-            desc = desc[:]
-            desc.extend(self.descriptionSuffix)
-        return desc
-
     def computeSourceRevision(self, changes):
         """Each subclass must implement this method to do something more
         precise than -rHEAD every time. For version control systems that use
@@ -198,6 +189,7 @@ class Source(LoggingBuildStep, CompositeStepMixin):
         self.checkoutDelay value."""
         return None
 
+    @defer.inlineCallbacks
     def applyPatch(self, patch):
         patch_command = ['patch', '-p{}'.format(patch[0]), '--remove-empty-files',
                          '--force', '--forward', '-i', '.buildbot-diff']
@@ -207,16 +199,13 @@ class Source(LoggingBuildStep, CompositeStepMixin):
                                                logEnviron=self.logEnviron)
 
         cmd.useLog(self.stdio_log, False)
-        d = self.runCommand(cmd)
+        yield self.runCommand(cmd)
+        if cmd.didFail():
+            raise buildstep.BuildStepFailed()
+        return cmd.rc
 
-        @d.addCallback
-        def evaluateCommand(_):
-            if cmd.didFail():
-                raise buildstep.BuildStepFailed()
-            return cmd.rc
-        return d
-
-    def patch(self, _, patch):
+    @defer.inlineCallbacks
+    def patch(self, patch):
         diff = patch[1]
         root = None
         if len(patch) >= 3:
@@ -230,36 +219,33 @@ class Source(LoggingBuildStep, CompositeStepMixin):
             if workdir_root_abspath.startswith(workdir_abspath):
                 self.workdir = workdir_root
 
-        d = self.downloadFileContentToWorker('.buildbot-diff', diff)
-        d.addCallback(
-            lambda _: self.downloadFileContentToWorker('.buildbot-patched', 'patched\n'))
-        d.addCallback(lambda _: self.applyPatch(patch))
+        yield self.downloadFileContentToWorker('.buildbot-diff', diff)
+        yield self.downloadFileContentToWorker('.buildbot-patched', 'patched\n')
+        yield self.applyPatch(patch)
         cmd = remotecommand.RemoteCommand('rmdir',
                                           {'dir': self.build.path_module.join(self.workdir,
                                                                               ".buildbot-diff"),
                                            'logEnviron': self.logEnviron})
         cmd.useLog(self.stdio_log, False)
-        d.addCallback(lambda _: self.runCommand(cmd))
+        yield self.runCommand(cmd)
 
-        @d.addCallback
-        def evaluateCommand(_):
-            if cmd.didFail():
-                raise buildstep.BuildStepFailed()
-            return cmd.rc
-        return d
+        if cmd.didFail():
+            raise buildstep.BuildStepFailed()
+        return cmd.rc
 
     def sourcedirIsPatched(self):
         d = self.pathExists(
             self.build.path_module.join(self.workdir, '.buildbot-patched'))
         return d
 
-    def start(self):
-        if self.notReally:
-            log.msg("faking {} checkout/update".format(self.name))
-            self.step_status.setText(["fake", self.name, "successful"])
-            self.addCompleteLog("log",
-                                "Faked {} checkout/update 'successful'\n".format(self.name))
-            return SKIPPED
+    @defer.inlineCallbacks
+    def run(self):
+        if getattr(self, 'startVC', None) is not None:
+            msg = 'Old-style source steps are no longer supported. Please convert your custom ' \
+                  'source step to new style (replace startVC with run_vc and convert all used ' \
+                  'old style APIs to new style). Please consider contributing the source step to ' \
+                  'upstream BuildBot so that such migrations can be avoided in the future.'
+            raise NotImplementedError(msg)
 
         if not self.alwaysUseLatest:
             # what source stamp would this step like to use?
@@ -284,14 +270,13 @@ class Source(LoggingBuildStep, CompositeStepMixin):
                 # root is optional.
                 patch = s.patch
                 if patch:
-                    self.addCompleteLog("patch", bytes2unicode(patch[1]))
+                    yield self.addCompleteLog("patch", bytes2unicode(patch[1]))
             else:
                 log.msg("No sourcestamp found in build for codebase '{}'".format(self.codebase))
-                self.step_status.setText(
-                    ["Codebase", '{}'.format(self.codebase), "not", "in", "build"])
-                self.addCompleteLog("log",
-                                    "No sourcestamp found in build for codebase '{}'".format(
-                                            self.codebase))
+                self.descriptionDone = "Codebase {} not in build".format(self.codebase)
+                yield self.addCompleteLog("log",
+                                          "No sourcestamp found in build for codebase '{}'".format(
+                                               self.codebase))
                 return FAILURE
 
         else:
@@ -299,5 +284,5 @@ class Source(LoggingBuildStep, CompositeStepMixin):
             branch = self.branch
             patch = None
 
-        self.startVC(branch, revision, patch)
-        return None
+        res = yield self.run_vc(branch, revision, patch)
+        return res
