@@ -84,19 +84,19 @@ class Tests(RunFakeMasterTestCase):
             'protocols': {'null': {}},
             'multiMaster': True,
         }
-        master = yield self.getMaster(config_dict)
+        yield self.setup_master(config_dict)
         builder_ids = [
-            (yield master.data.updates.findBuilderId('testy-1')),
-            (yield master.data.updates.findBuilderId('testy-2')),
+            (yield self.master.data.updates.findBuilderId('testy-1')),
+            (yield self.master.data.updates.findBuilderId('testy-2')),
         ]
 
         started_builds = []
-        yield master.mq.startConsuming(
+        yield self.master.mq.startConsuming(
             lambda key, build: started_builds.append(build),
             ('builds', None, 'new'))
 
         # Trigger a buildrequest
-        bsid, brids = yield master.data.updates.addBuildset(
+        bsid, brids = yield self.master.data.updates.addBuildset(
             waited_for=False,
             builderids=builder_ids,
             sourcestamps=[
@@ -138,19 +138,19 @@ class Tests(RunFakeMasterTestCase):
             'protocols': {'null': {}},
             'multiMaster': True,
         }
-        master = yield self.getMaster(config_dict)
+        yield self.setup_master(config_dict)
         builder_ids = [
-            (yield master.data.updates.findBuilderId('testy-1')),
-            (yield master.data.updates.findBuilderId('testy-2')),
+            (yield self.master.data.updates.findBuilderId('testy-1')),
+            (yield self.master.data.updates.findBuilderId('testy-2')),
         ]
 
         started_builds = []
-        yield master.mq.startConsuming(
+        yield self.master.mq.startConsuming(
             lambda key, build: started_builds.append(build),
             ('builds', None, 'new'))
 
         # Trigger a buildrequest
-        bsid, brids = yield master.data.updates.addBuildset(
+        bsid, brids = yield self.master.data.updates.addBuildset(
             waited_for=False,
             builderids=builder_ids,
             sourcestamps=[
@@ -182,9 +182,71 @@ class Tests(RunFakeMasterTestCase):
             'multiMaster': True,
         }
 
-        yield self.getMaster(config_dict)
+        yield self.setup_master(config_dict)
 
         self.assertIs(worker.machine, machine)
 
+    @defer.inlineCallbacks
+    def test_worker_reconfigure_with_new_builder(self):
+        """
+        Checks if we can successfully reconfigure if we add new builders to worker.
+        """
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder1",
+                              workernames=['local1'],
+                              factory=BuildFactory()),
+            ],
+            'workers': [self.createLocalWorker('local1', max_builds=1)],
+            'protocols': {'null': {}},
+            # Disable checks about missing scheduler.
+            'multiMaster': True,
+        }
+        yield self.setup_master(config_dict)
+
+        config_dict['builders'] += [
+            BuilderConfig(name="builder2",
+                          workernames=['local1'],
+                          factory=BuildFactory()),
+        ]
+        config_dict['workers'] = [self.createLocalWorker('local1', max_builds=2)]
+
+        # reconfig should succeed
+        yield self.reconfig_master(config_dict)
+
+    @defer.inlineCallbacks
+    def test_worker_os_release_info_roundtrip(self):
+        """
+        Checks if we can successfully get information about the platform the worker is running on.
+        This is very similar to test_worker_comm.TestWorkerComm.test_worker_info, except that
+        we check details such as whether the information is passed in correct encoding.
+        """
+        worker = self.createLocalWorker('local1')
+
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder1",
+                              workernames=['local1'],
+                              factory=BuildFactory()),
+            ],
+            'workers': [worker],
+            'protocols': {'null': {}},
+            # Disable checks about missing scheduler.
+            'multiMaster': True,
+        }
+        yield self.setup_master(config_dict)
+
+        props = worker.info
+
+        from buildbot_worker.base import BotBase
+
+        expected_props_dict = {}
+        BotBase._read_os_release(BotBase.os_release_file, expected_props_dict)
+
+        for key, value in expected_props_dict.items():
+            self.assertTrue(isinstance(key, str))
+            self.assertTrue(isinstance(value, str))
+            self.assertEqual(props.getProperty(key), value)
+
     if RemoteWorker is None:
         skip = "buildbot-worker package is not installed"
