@@ -30,8 +30,8 @@ from buildbot import master
 from buildbot import monkeypatches
 from buildbot.db import exceptions
 from buildbot.interfaces import IConfigLoader
+from buildbot.test import fakedb
 from buildbot.test.fake import fakedata
-from buildbot.test.fake import fakedb
 from buildbot.test.fake import fakemq
 from buildbot.test.fake.botmaster import FakeBotMaster
 from buildbot.test.util import dirs
@@ -66,7 +66,8 @@ class InitTests(unittest.SynchronousTestCase):
 
     def test_configfile_default(self):
         """
-        If neither configfile nor config_loader are specified, The default config_loader is a `FileLoader` pointing at `"master.cfg"`.
+        If neither configfile nor config_loader are specified, The default config_loader is a
+        `FileLoader` pointing at `"master.cfg"`.
         """
         m = master.BuildMaster(".", reactor=reactor)
         self.assertEqual(m.config_loader, config.FileLoader(".", "master.cfg"))
@@ -89,8 +90,6 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin,
         # patch out a few other annoying things the master likes to do
         self.patch(monkeypatches, 'patch_all', lambda: None)
         self.patch(signal, 'signal', lambda sig, hdlr: None)
-        # XXX temporary
-        self.patch(master, 'Status', lambda master: mock.Mock())
 
         master.BuildMaster.masterHeartbeatService = mock.Mock()
         self.master = master.BuildMaster(
@@ -98,12 +97,12 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin,
         self.master.sendBuildbotNetUsageData = mock.Mock()
         self.master.botmaster = FakeBotMaster()
         self.db = self.master.db = fakedb.FakeDBConnector(self)
-        self.db.setServiceParent(self.master)
+        yield self.db.setServiceParent(self.master)
         self.mq = self.master.mq = fakemq.FakeMQConnector(self)
-        self.mq.setServiceParent(self.master)
+        yield self.mq.setServiceParent(self.master)
         self.data = self.master.data = fakedata.FakeDataConnector(
             self.master, self)
-        self.data.setServiceParent(self.master)
+        yield self.data.setServiceParent(self.master)
 
     def tearDown(self):
         return self.tearDownDirs()
@@ -167,6 +166,9 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin,
         d = self.master.stopService()
 
         self.assertFalse(d.called)
+
+        # master must only send shutdown once builds are completed
+        self.assertTrue(self.master.data.updates.thisMasterActive)
         self.master.botmaster.shutdownDeferred.callback(None)
         self.assertTrue(d.called)
 
