@@ -13,6 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
+from twisted.internet import defer
+
 from buildbot.util import service
 from buildbot.util import subscription
 
@@ -28,8 +30,7 @@ class Connection:
         self.master = master
         self.worker = worker
         name = worker.workername
-        self._disconnectSubs = subscription.SubscriptionPoint(
-            "disconnections from %s" % name)
+        self._disconnectSubs = subscription.SubscriptionPoint("disconnections from {}".format(name))
 
     # This method replace all Impl args by their Proxy protocol implementation
     def createArgsProxies(self, args):
@@ -42,9 +43,15 @@ class Connection:
         return newargs
     # disconnection handling
 
+    def waitShutdown(self):
+        return defer.succeed(None)
+
     def notifyOnDisconnect(self, cb):
         return self._disconnectSubs.subscribe(cb)
 
+    def waitForNotifyDisconnectedDelivered(self):
+        return self._disconnectSubs.waitForDeliveriesToFinish()
+
     def notifyDisconnected(self):
         self._disconnectSubs.deliver()
 
