@@ -90,8 +90,8 @@ class ReferenceableProxy(pb.Referenceable):
         assert isinstance(impl, self.ImplClass)
         self.impl = impl
 
-    def __getattr__(self, default=None):
-        return getattr(self.impl, default)
+    def __getattr__(self, name):
+        return getattr(self.impl, name)
 
 
 # Proxy are just ReferenceableProxy to the Impl classes
@@ -115,10 +115,10 @@ class _NoSuchMethod(Exception):
 def _wrapRemoteException():
     try:
         yield
-    except pb.RemoteError as ex:
-        if ex.remoteType in (b'twisted.spread.flavors.NoSuchMethod',
-                             'twisted.spread.flavors.NoSuchMethod'):
-            raise _NoSuchMethod(ex)
+    except pb.RemoteError as e:
+        if e.remoteType in (b'twisted.spread.flavors.NoSuchMethod',
+                            'twisted.spread.flavors.NoSuchMethod'):
+            raise _NoSuchMethod(e) from e
         raise
 
 
